	/*
Barry Taylor's Better Volume Indicator as described at
http://emini-watch.com/free-stuff/volume-indicator/
Coded for NinjaTrader by Alex Matulich, 2009-10-08, Unicorn Research Corporation.
Additions and Amendments by TradingCoders, Oct 2010, March 2011. www.TradingCoders.com

This indicator optionally uses True Range rather than Range, it plots an exponential
moving average of the volume instead of simple moving average for efficiency,
and it optionally paints the price bars when BetterVolume indicates something
other than the normal color.

Watch for these at different stages of the market:

                  | Bottom | Start |Continue|  Top   |  Start  |Continue
                  |        |uptrend|uptrend |        |downtrend|downtrend
------------------+--------+-------+--------+--------+---------+---------
Volume Climax Up  |        |  Yes  |        |  Yes   |         |   Yes     red
Volume Climax Down|  Yes   |       |  Yes   |        |  Yes    |           white/black
High Volume Churn |  Yes   |       |        |  Yes   |         |           green/blue
Climax + Hi Churn |  Yes   |       |        |  Yes   |         |           magenta
Low Volume        |  Yes   |       |  Yes   |  Yes   |         |   Yes     yellow
-------------------------------------------------------------------------

The original Tradestation
code makes use of up volume and down volume data for intraday bars, and for daily
bars it calculates up volume and down volume from the relative positions of open,
high, low, and close. 
A new model in NT7 has been created to emulate this.
If necessary we use the latter TradeStation method here in absence of 
Tick data.
*/
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Better Volume indicator from emini-watch.com
    /// </summary>
    [Description("Better Volume indicator from emini-watch.com")]
    [Gui.Design.DisplayName("_BetterVolume")]
    public class BetterVolume : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int lookback = 20; // Default setting for Lookback
			private bool use2bars = true; // Default setting for Use2Bars
			private bool paintbars = true; // Default setting for Paintbars
			private bool useTrueRange = true; // Default setting for UseTrueRange
			private bool showTickModelBars = false;		// REMOVED FROM VIEW BY REQUEST
			private bool useTickVolume = true;
			private string password = "";
		
        // User defined variables (add any user defined variables below)
            private int barwidth = 2;
			private DataSeries[] value;
			private double vwt;
			private bool eod = true;
			private int bc = 0;
		
			// upticks/downticks
			bool lastWasUpTick = false;
			double lastLegacyPrice = 0;
			private DateTime lastLegacyTickTime = DateTime.MinValue;
			int lastCurrentBar = -1;
			int barUpTicks = 0;
			int barDnTicks = 0;
			double barUpVol = 0;
			double barDnVol = 0;
			int lastUpTicks = 0;
			int lastDnTicks = 0;
			double lastUpVol = 0;
			double lastDnVol = 0;
		
			private double thisBarBUYVol = 0;
			private double thisBarSELLVol = 0;
		
			private bool bInit = false;
			private bool useHLCModel = true;	// default model, used until valid Bid/Ask data is available.
			private bool skipNextTickTick = false;	// in realtime we need to read a tick early, from BIP[0]. Thus we need to skip it when it 'actually' happens.
			private bool hadFirstRealtimeTick = false; // helps realise when the first realtime tick is
		
		/// <summary>
		/// communicates the signal status, eg ClimaxUp, HiChurn etc.
		/// No state = 0
		/// ClimaxUp state = 1
		/// ClimaxDown state = 2
		/// HiChurn state = 3
		/// LowVol state = 4
		/// ClimaxChurn state = 5
		/// </summary>
			private IntSeries	signals;	// communicates the signal status, eg ClimaxUP, HiChurn etc.

		// Programmer's presets
		
        #endregion

		
        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {			
			this.Name = "_BetterVolume";
			int hilight=barwidth+1;
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Silver), barwidth), PlotStyle.Bar, "VBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Goldenrod), hilight), PlotStyle.Bar, "LowVol"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Red), hilight), PlotStyle.Bar, "ClimaxUp"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Black), hilight), PlotStyle.Bar, "ClimaxDown"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Green), hilight), PlotStyle.Bar, "HiChurn"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Magenta), hilight), PlotStyle.Bar, "ClimaxChurn"));
			Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Red), 1), PlotStyle.Line, "VolEMA"));

            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;

			value = new DataSeries[23];
			for (int i = 1; i <= 22; ++i) value[i] = new DataSeries(this); // value1 to value22
			vwt = 2.0 / (5.0 * lookback + 1.0); // bar average period is 5*lookback
			
			eod = (BarsPeriods[0].Id == PeriodType.Day || BarsPeriods[0].Id == PeriodType.Week || BarsPeriods[0].Id == PeriodType.Month || BarsPeriods[0].Id == PeriodType.Year);
			bool exotic = (BarsPeriods[0].Id == PeriodType.Renko || BarsPeriods[0].Id == PeriodType.Kagi || BarsPeriods[0].Id == PeriodType.LineBreak || BarsPeriods[0].Id == PeriodType.PointAndFigure);
					
			if (useTickVolume)
			{
				if (	exotic || 
						((!eod) && 
							(	BarsPeriods[0].Id != PeriodType.Minute || (BarsPeriods[0].Id == PeriodType.Minute && BarsPeriods[0].Value < 60))
						)
					)
				{
					Add(PeriodType.Tick, 1);	// legacy
					//Print(this.Name+" adding Tick stream");
				}
				else
				{
					Add(PeriodType.Minute,1);	// broader brush for large timeframes
					//Print(this.Name+" adding Minute stream");
				}
			}
			
			signals = new IntSeries(this);
        }

		


        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (BarsInProgress == 0 && (CurrentBar <= 5 || (!bInit)))
			{
				//range = High[0] - Low[0];
				eod = (Bars.Period.Id == PeriodType.Day || Bars.Period.Id == PeriodType.Week || Bars.Period.Id == PeriodType.Month || Bars.Period.Id == PeriodType.Year);
				
				if (!bInit)
				{
					// do-once initialise				
				if (!CalculateOnBarClose)	
						DrawTextFixed("close error","CalculateOnBarClose required for "+this.Name,TextPosition.Center,Color.Black,new Font("Arial",12),Color.Transparent,Color.Yellow,9);
					else							
						bInit = true;	
				}
				
				if (bInit)
					VolEMA.Set(Volume[0]);
				
				return;
			
			}
			else if (!bInit)
				return; // do nothing until this is done
			
			// get our latest data from our other streams 
			 LegacyUpDownTickModel();	// call to legacy model, which calculates its own Volume count and Tick count.
				
			if (BarsInProgress > 0)
				return;	// nothing else in OnBarUpdate()
			

			// start of legacy BetterVolume >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
			double range, netvol;
			bool cond1, cond2, cond3, cond4, cond5, cond6, cond7, cond8, cond9, cond10,
				cond11=false, cond12=false, cond13=false, cond14=false, cond15=false, cond16=false, cond17=false, cond18=false, cond19=false, cond20=false,
				hiclose, loclose;
			
			
			range = useTrueRange ?
					Math.Max(High[0], Close[1]) - Math.Min(Low[0], Close[1]) // use true range
					: High[0] - Low[0]; // use normal range
			range = rnd(range);
			
			
			// CALL TO GET DATA (using various models)
			GetData(range); // -----------------------

			
			netvol = value[1][0] - value[2][0];
			//value[3][0] = Volume[0]; //Math.Abs(value[1][0]+value[2][0]);
			//Print("\t value[3][0] is "+value[3][0]);
			value[4][0] = value[1][0]*range;
			value[5][0] = netvol*range;
			value[6][0] = value[2][0]*range;
			value[7][0] = -netvol*range;
			if (range != 0.0) 
			{
				value[8][0] = value[1][0]/range;
				value[9][0] = netvol/range;
				value[10][0] = value[2][0]/range;
				value[11][0] = -netvol/range;
				value[12][0] = value[3][0]/range;
			}
			else // TradingCoders addition - avoid non-set values
			{
				for (int i = 8; i<=12; i++) value[i][0] = 0;
			}
			if (use2bars) 
			{
				double hh = MAX(High, 2)[0], ll = MIN(Low, 2)[0];
				range = useTrueRange ? Math.Max(hh, Close[2]) - Math.Min(ll, Close[2]) // use true range
					: hh - ll; // use normal range
				value[13][0] = value[3][0]+value[3][1];
				value[14][0] = (value[1][0]+value[1][1])*range;
				netvol = value[1][0] + value[1][1] - value[2][0] - value[2][1];
				value[15][0] = netvol*range;
				value[16][0] = (value[2][0]+value[2][1])*range;
				value[17][0] = -netvol*range;
				if (range != 0.0) 
				{
					value[18][0] = (value[1][0]+value[1][1])/range;
					value[19][0] = netvol/range;
					value[20][0] = (value[2][0]+value[2][1])/range;
					value[21][0] = -netvol/range;
					value[22][0] = value[13][0]/range;
				} 
				else 
				{ // set to previous values
					for (int i = 18; i <= 22; ++i) value[i][0] = value[i][1];
				}
			}

			hiclose = (Close[0] > Open[0]);
			loclose = (Close[0] < Open[0]);
			cond1 = (value[3][0] == MIN(value[3],lookback)[0]);
			cond2 = (value[4][0] == MAX(value[4],lookback)[0] && hiclose);
			cond3 = (value[5][0] == MAX(value[5],lookback)[0] && hiclose);
			cond4 = (value[6][0] == MAX(value[6],lookback)[0] && loclose);
			cond5 = (value[7][0] == MAX(value[7],lookback)[0] && loclose);
			cond6 = (value[8][0] == MIN(value[8],lookback)[0] && loclose);
			cond7 = (value[9][0] == MIN(value[9],lookback)[0] && loclose);
			cond8 = (value[10][0] == MIN(value[10],lookback)[0] && hiclose);
			cond9 = (value[11][0] == MIN(value[11],lookback)[0] && hiclose);
			cond10 = (value[12][0] == MAX(value[12],lookback)[0]);
			if (use2bars) {
				hiclose = (Close[0] > Open[0] && Close[1] > Open[1]);
				loclose = (Close[0] < Open[0] && Close[1] < Open[1]);
				cond11 = (value[13][0] == MIN(value[13],lookback)[0]);
				cond12 = (value[14][0] == MAX(value[14],lookback)[0] && hiclose);
				cond13 = (value[15][0] == MAX(value[15],lookback)[0] && hiclose);
				cond14 = (value[16][0] == MAX(value[16],lookback)[0] && loclose);
				cond15 = (value[17][0] == MAX(value[17],lookback)[0] && loclose);
				cond16 = (value[18][0] == MIN(value[18],lookback)[0] && loclose);
				cond17 = (value[19][0] == MIN(value[19],lookback)[0] && loclose);
				cond18 = (value[20][0] == MIN(value[20],lookback)[0] && hiclose);
				cond19 = (value[21][0] == MIN(value[21],lookback)[0] && hiclose);
				cond20 = (value[22][0] == MAX(value[22],lookback)[0]);
			}

			// bc = bar color = index number of plot created in Initialize()

			bool sameday = eod ? true : (Time[0].Day == Time[1].Day);
			bc = 0; // default 0 (normal volume bar)
			if (cond1 || (cond11 && sameday)) bc = 1; //LowVol
			if (cond2 || cond3 || cond8 || cond9 || ((cond12 || cond13 || cond18 || cond19) && sameday)) bc = 2; //ClimaxUp
			if (cond4 || cond5 || cond6 || cond7 || ((cond14 || cond15 || cond16 || cond17) && sameday)) bc = 3; //ClimaxDown
			if (cond10 || (cond20 && sameday)) bc = 4; //Churn
			if (bc == 4 && (cond2 || cond3 || cond4 || cond5 || cond6 || cond7 || cond8 || cond9 || (cond12 || cond13 || cond14 || cond15 || cond16 || cond17 || cond18 || cond19) && sameday))
				bc = 5; //ClimaxChurn
			
			// put in Signals
			/// communicates the signal status, eg ClimaxUp, HiChurn etc.
			/// No state = 0
			/// ClimaxUp state = 1
			/// ClimaxDown state = 2
			/// HiChurn state = 3
			/// LowVol state = 4
			/// ClimaxChurn state = 5
			switch (bc) {
				case 0:
					signals.Set(0);	// no signal
					break;
				case 1:
					signals.Set(4); // Low Volume
					break;
				case 2:
					signals.Set(1); // Climax up
					break;
				case 3:
					signals.Set(2); // Climax down
					break;
				case 4:
					signals.Set(3); // Churn
					break;
				case 5:
					signals.Set(5); // ClimaxChurn
					break;
				default:
					signals.Set(0);
					break;
			}
			
			
			// plot stuff
			for (int a = 0; a<Values.Length; a++)
				Values[a].Reset();	// NT7 requires everything reset each time (if not CalculateOnBarClose)
			Values[bc].Set(Volume[0]); //draw the volume bar we found
			if (paintbars && bc > 0) 
				BarColor = Plots[bc].Pen.Color; // paint the price bar
			VolEMA.Set(vwt * (Volume[0] - VolEMA[1]) + VolEMA[1]); // plot average; faster than SMA
        }
		
		// --------------------------------------
		
		#region TickModel
		
		private void GetData(double range)
		{
			
			int AZ = (Historical || CalculateOnBarClose) ? 0 : 1; 
			
			// determine which model we can use
			if (useTickVolume)
			{
				if (lastLegacyTickTime <= Time[AZ+1] || eod)
				{
					if (!useHLCModel)					
						Print(this.Name+" Switching to HLC Model at "+Time[0]);
					useHLCModel = true;	// no bid ask data to use
				}
				else
				{
					if (useHLCModel)					
						Print(this.Name+" Switching to TickModel at "+Time[0]);
					useHLCModel = false;
				}
			}
			else
				useHLCModel = true;	// force ON if not using TickVolume
			
			// calculate up volume and down volume USING HIGH/LOW/CLOSE HISTORIC OR DAILY MODEL >>>>>>>>>>>>>>>
			if (useHLCModel)
			{
				if (range == 0.0)
					value[1][0] = value[2][0] = 0.5 * Volume[0];
				else 
				{
					if (Close[0] > Open[0])
						value[1][0] = range / (2.0*range + Open[0]-Close[0]) * Volume[0];
					else if (Close[0] < Open[0])
						value[1][0] = (range+Close[0]-Open[0]) / (2.0*range + Close[0]-Open[0]) * Volume[0];
					else value[1][0] = 0.5 * Volume[0];
					value[2][0] = Volume[0] - value[1][0];						
				}
				
				value[3][0] = Volume[0];
			} // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
			else
			{	// BID ASK DATA
				if (FirstTickOfBar && ((!Historical) && (!CalculateOnBarClose)))
				{
					//Print(this.Name+" Realtime BUY/SELL volume for bar ended "+Time[1]+" is "+thisBarBUYVol+", "+thisBarSELLVol+", and base Ninja volume is "+Volume[1]+" ("+(Volume[1]-(thisBarBUYVol+thisBarSELLVol))+")");
					thisBarBUYVol = 0;	// reset these at the start of each bar.
					thisBarSELLVol = 0;
				}
				
				
				value[1][0] = thisBarBUYVol;							// BUY VOLUME
				value[2][0] = thisBarSELLVol;							// SELL VOLUME	
				value[3][0] = Math.Abs(value[1][0]+value[2][0]);		// TOTAL VOLUME
				
				if (FirstTickOfBar && (Historical || CalculateOnBarClose))
				{
					//Print(this.Name+" BarClose BUY/SELL volume for bar ended "+Time[0]+" is "+thisBarBUYVol+", "+thisBarSELLVol+", and base Ninja volume is "+Volume[0]+" ("+(Volume[0]-(thisBarBUYVol+thisBarSELLVol))+")");
					thisBarBUYVol = 0;	// reset these at each bar.
					thisBarSELLVol = 0;
				}
				
				if (showTickModelBars)
					BackColor = Color.FromArgb(80,Color.PaleTurquoise);
			}
			
		}
		
		// -----------------------------------------
		
		
		#region Legacy updowntick model - 
		//
		/// <summary>
		/// pre NT7 model, using volume traded at or after an uptick, and at or after a downtick
		/// </summary>
		private void LegacyUpDownTickModel()
		{
			int bip = BarsArray.Length -1; // will be 1 when data is available, 0 otherwise
			if (BarsInProgress == bip)	// original legacy was 1, new tickmodel is 3
				{
					TickModelProcessor(bip);
					skipNextTickTick = false;
				}
			else if (BarsInProgress == 0)
			{
				if ((!Historical) 
					|| (BarsArray.Length>1 && BarsArray[1].Period.Id==PeriodType.Minute)
					) // do this historically with minutes, but only realtime for ticks
				{
					TickModelProcessor(bip);
					skipNextTickTick = true;
				}
				
				// reset upticks/dnticks
				if (CurrentBar > lastCurrentBar)
				{
					lastUpTicks = barUpTicks;	// Legacy variables
					lastDnTicks = barDnTicks;
					lastUpVol = barUpVol;
					lastDnVol = barDnVol;
					
					thisBarBUYVol = barUpVol;	// variables used in new code - same, just different names.
					thisBarSELLVol = barDnVol;
					
					barUpTicks = 0;
					barDnTicks = 0;
					barUpVol = 0;
					barDnVol = 0;
					
					lastCurrentBar = CurrentBar;
				}
			}
		}
		
		
		private void TickModelProcessor(int bip)
		{
			// avoid "object reference not set to an object"
			if (Closes[bip] == null)
				return;
			if (Volumes[bip] == null)
				return;
			
			lastLegacyTickTime = Times[bip][0];
			
			if (//(!Historical) && 
				skipNextTickTick 
				//&& BarsInProgress==bip 
				//&& Bars.Period.Id==PeriodType.Tick
				)
				return;
			if ((!Historical) && (!hadFirstRealtimeTick))
			{
				//Print("FIRST REALTIME TICK");
				hadFirstRealtimeTick = true;
			}
			//if (!Historical)	Print("\t["+BarsInProgress+"]"+lastLegacyTickTime.ToLongTimeString()+" "+Volumes[bip][0]);
			
			// update up ticks and down ticks
			if (Closes[bip][0] > lastLegacyPrice)
			{
				barUpTicks++;
				barUpVol += Volumes[bip][0];
				lastLegacyPrice = Closes[bip][0];
				lastWasUpTick = true;
			}
			else if (Closes[bip][0] < lastLegacyPrice)
			{
				barDnTicks++;
				barDnVol += Volumes[bip][0];
				lastLegacyPrice = Closes[bip][0];
				lastWasUpTick = false;
			}
			else
			{
				if (lastWasUpTick)
				{
					barUpTicks++;
					barUpVol += Volumes[bip][0];
				}
				else
				{
					barDnTicks++;
					barDnVol += Volumes[bip][0];
				}
			}
		}
		
		//
		#endregion

 
 		// ----------------------------
		
		#endregion
		
		#region misc
		
		
		private double rnd(double price)
		{
			return Instrument.MasterInstrument.Round2TickSize(price);
		}
		
		// ------------------------------------------
		
		public override string ToString() //formats chart label	
		{				
			return this.Name;			
		}				

		// ---------------------------------------------------------------------------------------------------------------
		private DateTime Now
		{
			get {
				DateTime now;
				if (Bars != null && Bars.MarketData != null &&
						Bars.MarketData.Connection.Options.Provider == Cbi.Provider.Replay) {
					now = Bars.MarketData.Connection.Now;
				}
				else {
					now = DateTime.Now;
				}
				return now;
			}
		}
		
		#endregion

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VBar
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowVol
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ClimaxUp
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ClimaxDown
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HiChurn
        {
            get { return Values[4]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ClimaxChurn
        {
            get { return Values[5]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VolEMA
        {
            get { return Values[6]; }
        }

		/// <summary>
		/// communicates the signal status, eg ClimaxUp, HiChurn etc.
		/// No state = 0
		/// ClimaxUp state = 1
		/// ClimaxDown state = 2
		/// HiChurn state = 3
		/// LowVol state = 4
		/// ClimaxChurn state = 5
		/// </summary>
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public IntSeries Signals
        {
            get { //Update();
				return signals; }
        }
		
        [Description("Volume bars to look back for min/max values")]
        [GridCategory("Parameters")]
        public int Lookback
        {
            get { return lookback; }
            set { lookback = Math.Max(2, value); }
        }

		[Description("True=use last 2 bars for calculations, false=use current bar")]
        [GridCategory("Parameters")]
        public bool Use2bars
        {
            get { return use2bars; }
            set { use2bars = value; }
        }

		[Description("True=show paintbars, false=leave price bars alone.")]
        [GridCategory("Parameters")]
        public bool Paintbars
        {
            get { return paintbars; }
            set { paintbars = value; }
        }

		[Description("True=use true range in calculations, false=use High-Low range.")]
        [GridCategory("Parameters")]
        public bool UseTrueRange
        {
            get { return useTrueRange; }
            set { useTrueRange = value; }
        }
		
//		[Description("True=Paint backcolor of indicator when able to use Bid/Ask volumes.")]
//        [GridCategory("Parameters")]
//        public bool ShowTickModelBars
//        {
//            get { return showTickModelBars; }
//            set { showTickModelBars = value; }
//        }
		
		[Description("True=Use Tick Volume. (when available.) Uses Minute data for 60m and higher timeframes.")]
        [GridCategory("Parameters")]
        public bool UseTickVolume
        {
            get { return useTickVolume; }
            set { useTickVolume = value; }
        }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BetterVolume[] cacheBetterVolume = null;

        private static BetterVolume checkBetterVolume = new BetterVolume();

        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        public BetterVolume BetterVolume(int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            return BetterVolume(Input, lookback, paintbars, use2bars, useTickVolume, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        public BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            if (cacheBetterVolume != null)
                for (int idx = 0; idx < cacheBetterVolume.Length; idx++)
                    if (cacheBetterVolume[idx].Lookback == lookback && cacheBetterVolume[idx].Paintbars == paintbars && cacheBetterVolume[idx].Use2bars == use2bars && cacheBetterVolume[idx].UseTickVolume == useTickVolume && cacheBetterVolume[idx].UseTrueRange == useTrueRange && cacheBetterVolume[idx].EqualsInput(input))
                        return cacheBetterVolume[idx];

            lock (checkBetterVolume)
            {
                checkBetterVolume.Lookback = lookback;
                lookback = checkBetterVolume.Lookback;
                checkBetterVolume.Paintbars = paintbars;
                paintbars = checkBetterVolume.Paintbars;
                checkBetterVolume.Use2bars = use2bars;
                use2bars = checkBetterVolume.Use2bars;
                checkBetterVolume.UseTickVolume = useTickVolume;
                useTickVolume = checkBetterVolume.UseTickVolume;
                checkBetterVolume.UseTrueRange = useTrueRange;
                useTrueRange = checkBetterVolume.UseTrueRange;

                if (cacheBetterVolume != null)
                    for (int idx = 0; idx < cacheBetterVolume.Length; idx++)
                        if (cacheBetterVolume[idx].Lookback == lookback && cacheBetterVolume[idx].Paintbars == paintbars && cacheBetterVolume[idx].Use2bars == use2bars && cacheBetterVolume[idx].UseTickVolume == useTickVolume && cacheBetterVolume[idx].UseTrueRange == useTrueRange && cacheBetterVolume[idx].EqualsInput(input))
                            return cacheBetterVolume[idx];

                BetterVolume indicator = new BetterVolume();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Lookback = lookback;
                indicator.Paintbars = paintbars;
                indicator.Use2bars = use2bars;
                indicator.UseTickVolume = useTickVolume;
                indicator.UseTrueRange = useTrueRange;
                Indicators.Add(indicator);
                indicator.SetUp();

                BetterVolume[] tmp = new BetterVolume[cacheBetterVolume == null ? 1 : cacheBetterVolume.Length + 1];
                if (cacheBetterVolume != null)
                    cacheBetterVolume.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheBetterVolume = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BetterVolume BetterVolume(int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            return _indicator.BetterVolume(Input, lookback, paintbars, use2bars, useTickVolume, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        public Indicator.BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            return _indicator.BetterVolume(input, lookback, paintbars, use2bars, useTickVolume, useTrueRange);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BetterVolume BetterVolume(int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            return _indicator.BetterVolume(Input, lookback, paintbars, use2bars, useTickVolume, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com
        /// </summary>
        /// <returns></returns>
        public Indicator.BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool use2bars, bool useTickVolume, bool useTrueRange)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BetterVolume(input, lookback, paintbars, use2bars, useTickVolume, useTrueRange);
        }
    }
}
#endregion
